package lcsb.mapviewer.api.projects.models.bioEntities;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.UserAccessException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.ISearchService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.view.AnnotationViewFactory;

@Transactional(value = "txManager")
public class BioEntitiesRestImpl extends BaseRestImpl {

	Logger								 logger	= Logger.getLogger(BioEntitiesRestImpl.class);

	@Autowired
	private IUserService	 userService;

	@Autowired
	private IModelService	 modelService;

	@Autowired
	private ISearchService searchService;

	@Autowired
	AnnotationViewFactory	 annotationViewFactory;

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	public List<Map<String, Object>> getClosestElementsByCoordinates(String projectId, String modelId, String token, Point2D coordinates, Integer count, String perfectMatch)
			throws UserAccessException, SecurityException {
		List<Map<String, Object>> resultMap = new ArrayList<>();

		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));

		Model submodel = model.getSubmodelById(modelId);

		List<Object> elements = searchService.getClosestElements(submodel, coordinates, count, perfectMatch.equalsIgnoreCase("true"));
		for (Object object : elements) {
			Map<String, Object> result = createMinifiedSearchResult(object);
			resultMap.add(result);
		}
		return resultMap;
	}

	/**
	 * @return the modelService
	 * @see #modelService
	 */
	public IModelService getModelService() {
		return modelService;
	}

	/**
	 * @param modelService
	 *          the modelService to set
	 * @see #modelService
	 */
	public void setModelService(IModelService modelService) {
		this.modelService = modelService;
	}

	/**
	 * @return the searchService
	 * @see #searchService
	 */
	public ISearchService getSearchService() {
		return searchService;
	}

	/**
	 * @param searchService
	 *          the searchService to set
	 * @see #searchService
	 */
	public void setSearchService(ISearchService searchService) {
		this.searchService = searchService;
	}

	public List<Map<String, Object>> getElementsByQuery(String projectId, String token, String modelId, String query, Integer maxElements, String perfectMatch)
			throws SecurityException {
		List<Map<String, Object>> resultMap = new ArrayList<>();

		Model model = modelService.getLastModelByProjectId(projectId, userService.getToken(token));
		
		Integer limit = Integer.valueOf(maxElements);
		boolean match = perfectMatch.equals("true");
		List<Object> elements = searchService.searchByQuery(model, query, limit, match);
		for (Object object : elements) {
			Map<String, Object> result = createMinifiedSearchResult(object);
			resultMap.add(result);
		}
		return resultMap;
	}

}
